#! /bin/env python
# unit tests for disposable module
#  $Id: test_disposable.py,v 1.10 2006/05/19 23:50:35 mwalfish Exp $

#
# Copyright (C) 2005-2006 Michael Walfish
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
import dqm.cryptkey_wrap.cryptkey as cryptkey
import dqm.crypt.disposable as disposable
import dqm.err as dqm_err
import datetime
import sys

# takes as input a datetime.date object
def one_week_prev(d):
    return d - datetime.timedelta(days=7)

if __name__ == "__main__":

    priv_key = cryptkey.gen_rabin_privkey(1024)
    pub_key  = priv_key.to_pubkey()
   
    dm = disposable.SimpleDisposable()
   
    today = datetime.datetime.utcnow()
    one_week_ago = one_week_prev(today)
    two_weeks_ago = one_week_prev(one_week_ago)
    dates = [today, one_week_ago, two_weeks_ago]
    
    this_week = today.isocalendar()[1]

    rc1 = dm.verify(dm.create(5, priv_key), pub_key)
    rc2, rc3, rc4 = [dm.verify( \
		      dm.create(5,priv_key,d.isocalendar()[1]), pub_key) \
		      for d in dates]
   

    DELIM = disposable.SimpleDisposable.DELIM

    dis_str_extra_field = DELIM.join(["234:17", 
				      str(this_week),
			              "1234341234124"])
    dis_str_bad_form = DELIM.join(["234*234",
			           str(this_week), 
				   "1234341234124"])
    dis_str_bad_form2 = "134adsasd213094sfj;dsfkcxz0932	4;lsafdkj;lfdsj"

    dis_str_good_faith = dm.create(5,priv_key)
    twiddled_char = dis_str_good_faith[-10:-9]
    if (twiddled_char == "A"): twiddled_char = "B"
    else: twiddled_char = "A"
    dis_str_changed_sig = dis_str_good_faith[:-10] + twiddled_char + \
			  dis_str_good_faith[-9:] 
    
    # this is the wrong private key; the signature should fail
    priv_key_wrong = cryptkey.gen_rabin_privkey(1024)
    dis_str_wrong_pub_key = dm.create(5,priv_key_wrong)

    rc5, rc6, rc7, rc8, rc9 = \
	[dm.verify(s, pub_key) \
		for s in [dis_str_extra_field, \
			  dis_str_bad_form,    \
			  dis_str_bad_form2,   \
			  dis_str_changed_sig,	\
			  dis_str_wrong_pub_key]]
    
    dis_str = dm.create(17, priv_key, weekstamp = 32)
    dis = dm.str_to_disposable(dis_str)
    # reconstructing sig 
    sig   = priv_key.sign(DELIM.join(dis_str.split(DELIM)[:-1]))

    rc10 = (dis['i']         == 17 and
            dis['weekstamp'] == 32)

    tests = [(lambda: rc1 == dqm_err.SUCCESS, "simple check?"),
	     (lambda: rc2 == dqm_err.SUCCESS, "another simple check?"),
	     (lambda: rc3 == dqm_err.SUCCESS, 
			     "weekstamp from last week succeeds?"),
	     (lambda: rc4 == dqm_err.BAD_WEEKSTAMP, "old weekstamp fails?"),
	     (lambda: rc5 == dqm_err.WRONG_FORMAT,
			     "recognizes an ill-formed disposable?"),
	     (lambda: rc6 == dqm_err.WRONG_FORMAT,
			     "recognizes another ill-formed disposable?"),
	     (lambda: rc7 == dqm_err.WRONG_FORMAT,
			     "recognizes yet another ill-formed disposable?"),
	     (lambda: rc8 == dqm_err.INVALID_DISPOSABLE_SIG,
			     "recognizes hacked signature?"),
	     (lambda: rc9 == dqm_err.INVALID_DISPOSABLE_SIG,
			     "recognizes signature of wrong key?"),
	     (lambda: rc10, "str_to_dis passes sanity?")]
	    
    for test,msg in tests:
	print msg,
	if not test(): print "FAILED"; sys.exit(1)
	print "passed"

    sys.exit(0)
