# 
# $Id: rpc_udp.py,v 1.27 2006/05/19 23:50:35 mwalfish Exp $
#
#
# Copyright (C) 2005-2006 Michael Walfish
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
import sys
import struct
from socket import *
import random
import dqm.rpc.events    as dqm_events
import dqm.rpc.udp_iface as udp_iface
import dqm.svc.RPCProto  as RPCProto
import dqm.logger

# save a lookup
pack         = struct.pack
unpack       = struct.unpack
struct_error = struct.error

# When the callback supplied as the constructor to the rpc_server object is 
# called, its argument is an 'rpc' object
class rpc:

    """the following members are public:
    	self.proc -- the particular procedure being invoked
        self.arg  -- the argument to that procedure
        self.addr -- the IP and port of the client (i.e., the caller)
        self.reply(res) -- returns 'res' to the caller
    """

    def __init__(self, xid, proc, arg, addr, network):
	self.xid = xid
	self.proc = proc
	self.arg  = arg
	self.addr = addr
	self.network = network

    def reply(self, res):
	
	# I was going to write a comment that this hack is brutal but,
	# actually, it's not that bad. If the code gets here, it means the
	# server wants to reply. There are two possibilities: either the
	# server is returning a number or it is returning a number and a
	# string.
	if isinstance(res,int):
	    fmt_string = '>7i'	

	elif isinstance(res,str):	
	    res_len = len(res)
	    fmt_string = '>6i%ds' % res_len

	else:
	    raise AssertionError, "'res' must be an int or a string"

	auth1 = auth2 = 0

	reply_buf = pack(fmt_string, 
			 self.xid,
			 RPCProto.REPLY,
			 RPCProto.MSG_ACCEPTED,
			 auth1,
			 auth2,
			 RPCProto.SUCCESS,
			 res)

	# XXX: hack to help with the experiment: if the source port is 2100,
	# then we reply on the UDP discard port. remove after experiment
	# is over.
#	if self.addr[1] == 2100: 
#	    self.network.queue_packet(reply_buf, (self.addr[0], 9))
#	else:
#	    self.network.queue_packet(reply_buf, self.addr)
	self.network.queue_packet(reply_buf, self.addr)

class rpc_server:

    def __init__(self, port, module, prog, vers, dispatcher, logger=None):

	if logger is None: logger = dqm.logger.stderr_logger()
	self.logger = logger

	self.network_iface = udp_iface.udp_dgrams(port, 
					          True, 
						  self.handle_packed_call,
						  logger=logger)
	self.module = module
	self.prog = prog
	self.vers = vers
	self.dispatcher = dispatcher
	self.valid_procs = self.module.programs[self.prog][self.vers].keys()

    def handle_packed_call(self, rcvd_tups, tup_len):

	# local variables for speed
	dispatcher = self.dispatcher
	our_prog   = self.prog
	our_vers   = self.vers
	our_valid  = self.valid_procs
	our_network = self.network_iface
	right_msg_type = RPCProto.CALL
	right_version  = RPCProto.RPC_VERSION

	for i in range(tup_len):

	    packet, addr = rcvd_tups[i]

	    # this is a brutal hack to get around the fact that python was
	    # spending too much time unpacking binary data. it relies on
	    # knowledge of our RPC format and on the similarity between put 
	    # and set and between get and test
	    try: 
		xid,msg_typ,rpcver,prog,vers,proc, \
		    auth1,auth2,auth3,auth4 = unpack('>10i', packet[0:40])
		
		if msg_typ == right_msg_type  and \
		   rpcver  == right_version   and \
		   prog    == our_prog        and \
		   vers    == our_vers        and \
		   proc in our_valid: 

		    arg = packet[40:]
		    r = rpc(xid, proc, arg, addr, our_network)
		    dispatcher(r)

	    except struct_error: 
		continue

	    # XXX: here, we *used* to reply to RPCs saying, "this is
	    # garbage." We're not going to do that here. That's partly for
	    # speed and partly because if the RPC is bad, it's the caller's
	    # fault (not exactly -- we might want to say that the version
	    # is old), but still.

CLI_TIMED_OUT      = 0
CLI_SENDING_ERROR  = 1
CLI_CALL_COMPLETED = 2

class rpc_call_req:
    def __init__(self,module,prog,vers,proc,timer_h,cb,cb_arg):
	self.module = module
	self.prog = prog
	self.vers = vers
	self.proc = proc
	self.timer_h = timer_h
	self.cb = cb
	self.cb_arg = cb_arg

class rpc_client:

    # set 'src_port' if you want to be able to control what source port
    # will be stamped in UDP packets.
    def __init__(self, logger=None, src_port=0):
	if logger is None: logger = dqm.logger.stderr_logger()
	self.logger = logger
	self.network = udp_iface.udp_dgrams(\
					src_port,     
			                False, # initially_reading = False
				        self.handle_packed_reply,
					logger=logger)
	self.xidmap = {}
	self.xid    = random.randint(0, sys.maxint/2)

    def null_call(self):
	pass

    def call(self, module, prog, vers, proc, arg, (host,port),
	     tmou_s, cb, cb_arg=None):
	"""'prog', 'vers', 'proc' refer to the particular RPC call the
	         caller of this function is making. 
	   'arg' is the argument to the RPC
	   'host' and 'port' indicate where the RPC server is running
	   'tmou_s' is the timeout in seconds (floating point number)
	   'cb' should be a function that accepts a tuple, t:
	       -->if t[0] is CLI_TIMED_OUT, the request failed. The packet send
	       event worked, but the receiver never got the packet or the
	       reply was dropped. t[1] is the empty string, ''.
	       -->if t[0] is CLI_SENDING_ERROR, there was a problem sending, 
	       e.g., the IP address in the packet was insensible. t[1] is a 
	       tuple giving the error code and the error string.
	       -->if t[0] is CLI_CALL_COMPLETED, the request succeeded. t[1] is 
	       the RPC result, whose type is given by the .x file.
	 It is an error to call this function with invalid 'module', 'prog', 
	 'proc', or 'vers'. 
	"""
	xid = self.xid
	self.xid = xid + 1

	arg_len = len(arg)
	# handle special case when we do a lookup on a key (arg_len will be 20).
	# otherwise, arg_len could be 44 + metastamp(len), 
	# and we do the silly format string thing.
	if arg_len == 20:   fmt_string = '>10i20s'
	else:               fmt_string = '>10i%ds' % arg_len

	auth1 = auth2 = auth3 = auth4 = 0

	packet_buf = pack(fmt_string,
			  xid,
			  RPCProto.CALL,
			  RPCProto.RPC_VERSION,
			  prog,
			  vers,
			  proc,
			  auth1,
			  auth2,
			  auth3,
			  auth4,
			  arg)
	
	self.network.queue_packet(packet_buf, (host,port))
	self.network.set_reading(True)

	timer_h = dqm_events.reg_fifo_timer_delay(tmou_s, self.nanny, xid)
	self.xidmap[xid] = rpc_call_req(module,prog,vers,proc,\
					     timer_h,cb,cb_arg)

    def nanny(self, xid):
	# Timeout occurred. delete the xid from the map and tell the user.
	req = self.xidmap[xid]
	self.del_xid(xid)
	req.cb(req.cb_arg, (CLI_TIMED_OUT, '') )

    def del_xid(self, xid):
	del self.xidmap[xid]
	if len(self.xidmap) == 0:
	    self.network.set_reading(False)

    def send_done(self, xid, ret):

	# We don't tell callers when their requests were sent
	# successfully.
	if ret[0]: return

	req = self.xidmap[xid]
	self.del_xid(xid)
	dqm_events.unreg_timer(req.timer_h)
	req.cb(req.cb_arg, (CLI_SENDING_ERROR, ret[1]) )

    def handle_packed_reply(self, rcvd_tups, tup_len):

	# local variables, for speed
	our_map = self.xidmap
	our_del_func = self.del_xid
	timer_unreg_func = dqm_events.unreg_timer
	correct_typ    = RPCProto.REPLY
	correct_status = RPCProto.MSG_ACCEPTED
	correct_rc     = RPCProto.SUCCESS

	for i in range(tup_len):
	    packet, addr = rcvd_tups[i]
	    try:

		# brutal hack. what these next lines of code do is:
		# if the return value is an integer, extract the integer in
		# the fastest way. the return value begins at byte 16.
		if len(packet) == 28:
		    xid,typ,status,auth1,auth2,acc_st,res = \
				    unpack('>7i', packet)
		else:
		    xid,typ,status,auth1,auth2,acc_st = \
				    unpack('>6i', packet[0:24])
		    res = packet[24:]

		if typ    != correct_typ or \
		   status != correct_status or\
		   acc_st != correct_rc:
		    logmsg = 'Reply to RPC contained error (1)'
		    self.logger.log(logmsg)
		    continue

	    except struct_error:

		logmsg = 'Reply to RPC contained error (2)'  
		self.logger.log(logmsg)
	    
		# DON'T let the user know. It's possible this was a stray
		# packet. Our drop-dead deadline is the timeout. Wait until
		# then before telling the RPC caller that anything is amiss.
		# No need for the caller to know that a
		# possibly-spurious reply to its request arrived.

	    else:

		# successful unpack. now let's see if the message was accepted
		# and if the xid is one we know about.  If we get here, then
		# message was accepted and reply should be well-formed.
		if our_map.has_key(xid):
		    req = our_map[xid]
		    our_del_func(xid)
		    timer_unreg_func(req.timer_h)

		    req.cb(req.cb_arg, (CLI_CALL_COMPLETED, res))
		#else:
		#   logmsg = 'got packet for xid %d but no entry for it' % xid
		#   self.logger.log(logmsg)
    
