#! /bin/env python
#
# $Id: test_db.py,v 1.9 2006/05/19 23:50:35 mwalfish Exp $
#
#  test routines for the pool of disposables that is backed by an on-disk
#  database

#
# Copyright (C) 2005-2006 Michael Walfish
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
import os
import os.path
import random
import dqm.stamp_pool.db as db_pool
from dqm.util.convert import *
import sys

if __name__ == "__main__":
 
    parent_dir = '/tmp/'
    db_name = 'mr.database'
    quota1 = 50
    quota2 = 0x00010000	 # for endian-testing, we want quota > 2^16 - 1

    dir1 = parent_dir + 'qwerty' + str(random.random())
    dir2 = parent_dir + 'asdf' + str(random.random())
    dir3 = parent_dir + 'ghijk' + str(random.random())
    for d in [dir1, dir2, dir3]:
	os.mkdir(d)

    dpool1 = db_pool.create_new(quota1, dir1, db_name)
    rc1 = dpool1 is not None
    dpool1.close()
    dpool2 = db_pool.create_from_file(dir1, db_name)
    # big-ass db
    dpool4 = db_pool.create_new(quota2, dir2, db_name)
    dpool5 = db_pool.create_new(quota1, dir3, db_name)

    def a_test(dpool,q):
	l = {} 
	for i in range(q):
	    r = dpool.consume_disposable()
	    if l.has_key(r): return False
	    l[r] = ""

	# checks
	if dpool.num_left() != 0: return False 

	r = dpool.consume_disposable()
	if r != -1: return False

	return True

    def b_test(dpool,d,fn,q):
	stop = q / 2
	l = {}
	for i in range(stop):
	    r = dpool.consume_disposable()
	    l[r] = ""

	dpool.close()
	reborn = db_pool.create_from_file(d, fn)
	
	for i in range(stop,q):
	    r = reborn.consume_disposable()
	    if l.has_key(r): return False
	    l[r] = ""

	if reborn.num_left() != 0: return False
	if reborn.get_quota() != q: return False
	
	r = reborn.consume_disposable()
	if r != -1: return False

	reborn.close()
	del reborn
	return True	

    tests = [(lambda: rc1, "simple check?"),
#	     (lambda: dpool3 == dpool2, "another sanity check?"),
	     (lambda: a_test(dpool2,quota1), "non-duplicates?"),
	     (lambda: a_test(dpool4,quota2), "big pool works?"),
	     (lambda: b_test(dpool5,dir3,db_name,quota1), 
		"write out and read in works?")]

    failure = False
    for test,msg in tests:
	print msg,
	if not test(): print "FAILED"; failure = True; break 
	else: print "passed"
	
    del dpool1
    del dpool2
    del dpool4
    del dpool5

    def cleanup(d): 
	for f in os.listdir(d): os.remove(os.path.join(d,f))
	os.rmdir(d)

    cleanup(dir1)
    cleanup(dir2)
    cleanup(dir3)

    if failure: sys.exit(1)
    else: sys.exit(0)
