/* 
 * test case to demonstrate strange behavior of DB locking function.
 *   Michael Walfish, 16 Feb. 2004
 */
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <db.h>
#include <assert.h>
#include <string.h>

#define OUR_DB_DIR		    "db_dir"
#define OUR_DB_NAME		    "test_db"
#define OUR_DBENV_FLAGS   (DB_INIT_LOCK | DB_INIT_MPOOL | DB_CREATE | DB_INIT_LOG)
#define OUR_DB_FLAGS       (DB_CREATE)
#define OUR_DB_REC_LEN	    4

#define ERR_CHECK(rc, ckpt)    \
    if (rc != 0) { \
	printf("error. ckpt: %s. rc: %d. msg: %s.\n", \
	       ckpt, rc, db_strerror(rc)); \
	exit(0); \
    } \

/* 
 * simple program -- the intent is to run two of these at once to see
 * whether the locker does what it is supposed to (namely letting the
 * second locker proceed only after the first locker has released the
 * lock).
 */
int main(int argc, char** argv)
{
    int rc;
    int lid;
    DB_ENV* dbe;
    DB* db;
    DB_LOCK lock;
    DBT thang;
    char* lock_name = "some_name_for_a_lock";
    
    thang.data = lock_name;
    thang.size = strlen(lock_name);

    mkdir(OUR_DB_DIR, 0777);
    
    rc = db_env_create(&dbe, 0);
    dbe->set_errfile(dbe, stdout);
    ERR_CHECK(rc, "a")
    rc = dbe->open(dbe, OUR_DB_DIR, OUR_DBENV_FLAGS, 0);
    ERR_CHECK(rc, "b")
    rc = db_create(&db, dbe, 0);
    ERR_CHECK(rc, "c")
    rc = db->open(db, NULL, OUR_DB_NAME, NULL, DB_QUEUE, OUR_DB_FLAGS, 0);
    ERR_CHECK(rc, "e")
    rc = dbe->lock_id(dbe, &lid);
    ERR_CHECK(rc, "f")

    printf("about to acquire lock. our id: %d\n", lid);
    rc = dbe->lock_get(dbe, lid, 0, &thang, DB_LOCK_WRITE, &lock);
    ERR_CHECK(rc, "g")
    printf("acquired lock\n");

    sleep(10);

    rc = dbe->lock_put(dbe, &lock);
    ERR_CHECK(rc, "h")
    printf("released lock\n");

    exit(0);
}
