/*
 * Copyright (C) 2005-2006 Michael Walfish
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#ifndef _CONFIG_H_
#define _CONFIG_H_

#include "async.h"
#include "qhash.h"
#include "dqm_node.h"

// Holds the config parameters for the DQM portal. The work of parsing
// the config file is done by the functions in ~libasync/async/parseopt.h.
// This class's main job is to respond to queries for various configuration
// parameters.
class config_params {

    public:

	config_params(str conf_file);
	~config_params();

	str identity()		  { return iden;    }

	str key_val_storage()     { return key_val_store; }
	int32_t read_size()     { return rd_size; }
	int32_t write_size()    { return wr_size; }

	u_int16_t portal_port()   { return portal_prt; }
	u_int16_t storer_port()   { return storer_prt; }
	u_int16_t internal_port() { return internal_prt; }

	int num_rings()     { return nrings; }
	int tree_degree()   { return d; }

	int r()                { return repl_factor;  }
	bool use_tree()           { return using_tree;    }
	int cache_size()       { return cache_entries; }

	uint32_t nindex_entries()  { return nindex; }

	const approved_list* get_storers()    { return &storers; }

    private:

	void parse_double(const char*, double*, vec<str>, str, bool*);
	void process_storer(vec<str> s, str loc, bool* errp);

	str iden;

	int32_t rd_size;
	int32_t wr_size;

	u_int16_t portal_prt;
	u_int16_t storer_prt;
	u_int16_t internal_prt;

	int nrings;
	int d;
	int cache_entries;
	uint32_t nindex;
	int repl_factor;
	bool using_tree;

	str key_val_store;

	approved_list storers;
};

#endif
