/*
 * Copyright (C) 2005-2006 Michael Walfish
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */
#include <stdio.h>
#include "log.h"

char buf[2048];

file_logger::file_logger(const char* logname) 
    : fd(-1)
{
   if ( (fd = open(logname, O_CREAT | O_APPEND | O_WRONLY, 0644)) == -1)
	fatal("couldn't open file %s: %m\n", logname);
}

file_logger::~file_logger() 
{
    close(fd);
}

void 
file_logger::log(const char* msg) 
{
    int num, rc;
    struct timeval tv;
    char timebuf[60];
    struct tm* tmp;

    gettimeofday(&tv, NULL);
    tmp = gmtime(reinterpret_cast<time_t*>(&tv.tv_sec));
    sprintf(timebuf, "%d-%.2d-%.2d %.2d:%.2d:%.2d.%.6ld", 
	    tmp->tm_year + 1900, 
	    tmp->tm_mon + 1, 
	    tmp->tm_mday, tmp->tm_hour, 
	    tmp->tm_min, tmp->tm_sec,
	    tv.tv_usec);

    num = sprintf(buf, "%s %s\n", timebuf, msg);
    if ((rc = write(fd, buf, num)) != num) 
	fatal("couldn't write to logfile: %m\n");
}

void
file_logger::sync() 
{
    fsync(fd);
}
