/*
 * Copyright (C) 2005-2006 Michael Walfish
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */
#include "async.h"
#include "parseopt.h"
#include "qhash.h"

qhash<str, str> signers;
qhash<str, str> storers;

void usage(char** argv)
{
    warnx ("usage: %s conf_file\n", argv[0]);
    exit(1);
}

void
process_signer(vec<str> s, str loc, bool* errp)
{
    if (s.size() != 3) {
	warn << loc << ": usage: trusted_signer SIGNER_NAME pubkey\n";
	*errp = true;
	return;
    }

    signers.insert(s[1], s[2]);
}

void
process_storer(vec<str> s, str loc, bool* errp)
{
    if (s.size() != 3) {
	warn << loc << ": usage: storer node_name"
	                " hostname:storer_port:internal_port\n";
	*errp = true;
	return;
    }

    storers.insert(s[1], s[2]);
}

void traverse_cb(const str& k, str* v)
{
    warnx("%s\n", k.cstr());
    warnx("%s\n", v->cstr());
}

void process(str cf)
{
    parseargs pa(cf);
    int line;
    vec<str> av;
    conftab ct;
    bool errors = false;

    u_int32_t portal_port;
    u_int32_t storer_port; 
    u_int32_t internal_port;
    str key_val_store;
    str identity;
    int num_rings;
    int tree_degree;

    ct.add("portal_port",     &portal_port, 0, 65535);
    ct.add("storer_port",     &storer_port, 0, 65535);
    ct.add("internal_port",   &internal_port, 0, 65535);
    ct.add("key_val_storage", &key_val_store);
    ct.add("trusted_signer",  wrap(process_signer));
    ct.add("identity",        &identity);
    ct.add("num_rings",       &num_rings, 0, ~(1 << 31));
    ct.add("tree_degree",     &tree_degree, 0, ~(1 << 31));
    ct.add("storer",          wrap(process_storer));
    ct.ignore("whos_approved_list");

    while (pa.getline(&av, &line)) {
	if (!ct.match(av, cf, line, &errors)) {
	    warn << cf << ":" << line << ": unknown config parameter\n";
	    errors = true;
	}
    }

    printf("pp: %u. sp: %u. ip: %u.\n", portal_port, storer_port,
    internal_port);

#if 0
    ptr<callback<void, const str&, str*> > cb = wrap(traverse_cb);
    signers.traverse(cb);
    storers.traverse(cb);
#endif

    for (qhash<str,str>::slot* s = storers.first(); s; s = storers.next(s)) 
	printf("%s: %s\n", s->key.cstr(), s->value.cstr());

}

int main(int argc, char** argv)
{
    if (argc != 2) 
	usage(argv);

    process(argv[1]);

    exit(0);
}
