/*
 * Copyright (C) 2005-2006 Michael Walfish
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */
#include "async.h"
#include "cryptkey.h"
#include "sha1.h"

int main(int argc, char** argv)
{
    if (argc != 3) {
	printf("usage: %s key-file salt\n", argv[0]);
	exit(0);
    }
    
    char dummybuf[800];
    sha1_hash(dummybuf, argv[2], strlen(argv[2]));
    str dummy(dummybuf, sha1::hashsize);

    ptr<privkey> privk = file_to_privkey(argv[1]);
    if (privk == NULL) 
	fatal << "couldn't open priv key file " << argv[1] << "\n";

    str sig = privk->sign(dummy);
    if (sig.cstr() == NULL) 
	fatal << "signature was NULL\n";

    ptr<pubkey> pubk = privk->to_pubkey();

    if (!pubk->verify(dummy,sig))
	warnx << "signature didn't verify!\n";

    return 0;
}
